@@ -3,7 +3,7 @@ use strict;
 
 package Devel::Chitin;
 
-use Devel::Chitin::Version;
+our $VERSION = '0.01';
 
 use Scalar::Util;
 use IO::File;
@@ -14,10 +14,6 @@ use Devel::Chitin::Stack;
 use Devel::Chitin::Location;
 use Devel::Chitin::SubroutineLocation;
 use Devel::Chitin::Exception;
-use Devel::Chitin::OpTree;
-
-use base 'Exporter';
-our @EXPORT_OK = qw( $VERSION );
 
 # lexicals shared between the interface package and the DB package
 my(%attached_clients,
@@ -26,9 +22,6 @@ my(%attached_clients,
    $is_initialized,
    @pending_eval,
    $current_location,
-   $previous_location,
-   @new_watch_exprs,
-   @watch_exprs,
 );
 sub attach {
     my $self = shift;
@@ -44,15 +37,11 @@ sub attach {
     return $self;
 }
 
-sub _turn_off_trace_if_not_needed {
-    $DB::trace = %trace_clients || @watch_exprs;
-}
-
 sub detach {
     my $self = shift;
     my $deleted = delete $attached_clients{$self};
     delete $trace_clients{$self};
-    _turn_off_trace_if_not_needed();
+    $DB::trace = %trace_clients ? 1 : 0;
     if ($deleted) {
         for (my $i = 0; $i < @attached_clients; $i++) {
             my $same = ref($self)
@@ -110,7 +99,10 @@ sub trace {
         } else {
             # turning it off
             delete $trace_clients{$class};
-            _turn_off_trace_if_not_needed();
+            if (%trace_clients) {
+                # No more clients requesting trace
+                $DB::trace = 0;
+            }
             $rv = 0;
         }
 
@@ -164,34 +156,6 @@ sub loaded_files {
     return map { substr($_,2) } @files; # remove the <_
 }
 
-sub add_watchexpr {
-    my($class, $expr) = @_;
-    $DB::trace = 1;
-    push @new_watch_exprs, { expr => $expr, client => $class, value => undef };
-}
-
-sub remove_watchexpr {
-    my($class, $expr) = @_;
-    my $deleted;
-
-    SEARCH:
-    foreach my $store ( \@watch_exprs, \@new_watch_exprs) {
-        for (my $i = 0; $i < @$store; $i++) {
-            if ($store->[$i]->{client} eq $class
-                and
-                $store->[$i]->{expr} eq $expr
-            ) {
-                $deleted = splice(@$store, $i, 1);
-                last SEARCH;
-            }
-        }
-    }
-
-    _turn_off_trace_if_not_needed();
-
-    return $deleted;
-}
-
 sub is_breakable {
     my($class, $filename, $line) = @_;
 
@@ -294,60 +258,6 @@ sub file_source {
     return *{$glob}{ARRAY};
 }
 
-my %optrees;
-sub next_statement {
-    my($class, $scopes) = @_;
-
-    my $loc = $class->current_location();
-    $loc = $class->_fixup_location_inside_eval($loc);
-    my $optree = $optrees{$loc->subroutine} ||= Devel::Chitin::OpTree->build_from_location($loc);
-
-    my $callsite = $loc->callsite;
-    my($last_cop, $current_op);
-    BREAKOUT:
-    for(1) {
-        $optree->walk_inorder(sub {
-            my $op = shift;
-            $last_cop = $op if ($op->isa('Devel::Chitin::OpTree::COP'));
-            if (${$op->op} == $callsite) {
-                $current_op = $op;
-                no warnings 'exiting';
-                last BREAKOUT;
-            }
-        });
-    }
-
-    while($last_cop && $scopes--) {
-        $last_cop = $last_cop->parent;
-    }
-
-    if ($last_cop) {
-        return $last_cop->sibling->deparse;
-    } else {
-        Carp::carp("Cannot find current opcode at $callsite in ".$loc->subroutine);
-        return '';
-    }
-}
-
-sub _fixup_location_inside_eval {
-    my($class, $loc) = @_;
-
-    if ($loc->subroutine eq '(eval)') {
-        my $stack = $class->stack->iterator;
-        my $frame;
-        for($frame = $stack->(); $frame; $frame = $stack->()) {
-            last if $frame->subroutine ne '(eval)';
-        }
-        if ($frame) {
-            return Devel::Chitin::Location->new(
-                        (map { $_ => $frame->$_ } qw(package filename line subroutine)),
-                        callsite => $loc->callsite
-                    );
-        }
-    }
-    return $loc;
-}
-
 ## Methods called by the DB core - override in clients
 
 sub init {}
@@ -357,13 +267,11 @@ sub cleanup {}
 sub notify_stopped {}
 sub notify_resumed {}
 sub notify_trace {}
-sub notify_trace_resumed {}
 sub notify_fork_parent {}
 sub notify_fork_child {}
 sub notify_program_terminated {}
 sub notify_program_exit {}
 sub notify_uncaught_exception {}
-sub notify_watch_expr {}
 
 sub _do_each_client {
     my($method, @args) = @_;
@@ -430,32 +338,6 @@ sub restore {
     ( $@, $!, $^E, $,, $/, $\, $^W ) = @saved;
 }
 
-sub _evaluate_watch_exprs {
-    EXPR:
-    foreach my $details ( @watch_exprs ) {
-        my($current_value) = _eval_in_program_context($details->{expr}, 1);
-        my $old_value = $details->{value};
-
-        if (@$current_value != @$old_value) {
-            $details->{client}->notify_watch_expr($previous_location, $details->{expr}, $old_value, $current_value);
-            $details->{value} = $current_value;
-            next EXPR;
-        }
-
-        for (my $i = 0; $i < @$current_value; $i++) {
-            no warnings 'uninitialized';
-            if ((defined($current_value->[$i]) xor defined($old_value->[$i]))
-                or
-                $current_value->[$i] ne $old_value->[$i]
-            ) {
-                $details->{client}->notify_watch_expr($previous_location, $details->{expr}, $old_value, $current_value);
-                $details->{value} = $current_value;
-                next EXPR;
-            }
-        }
-    }
-}
-
 sub is_breakpoint {
     my($package, $filename, $line) = @_;
 
@@ -499,9 +381,8 @@ sub is_breakpoint {
 sub _parent_stack_location {
     my($package, $filename, $line) = caller(1);
     my(undef, undef, undef, $subname) = caller(2);
-    my $callsite = Devel::Chitin::Location::get_callsite(2);
     $subname ||= 'MAIN';
-    return ($package, $filename, $line, $subname, $callsite);
+    return ($package, $filename, $line, $subname);
 }
 
 BEGIN {
@@ -510,13 +391,12 @@ BEGIN {
         my $pid = CORE::fork();
         return $pid unless $ready;
 
-        my($package, $filename, $line, $subname, $callsite) = _parent_stack_location();
+        my($package, $filename, $line, $subname) = _parent_stack_location();
         my $location = Devel::Chitin::Location->new(
             'package'   => $package,
             line        => $line,
             filename    => $filename,
             subroutine  => $subname,
-            callsite    => $callsite,
         );
 
         my $notify = $pid ? 'notify_fork_parent' : 'notify_fork_child';
@@ -533,15 +413,14 @@ BEGIN {
 # We could re-throw the die if $^S is 1
 $SIG{__DIE__} = sub {
     if (defined($^S) && $^S == 0) {
-        $in_debugger = 1;
         my $exception = $_[0];
         # It's interesting to note that if we pass an arg to caller() to
         # find out the offending subroutine name, then the line reported
-        # changes.  Instead of reporting the line the exception occurred
+        # changes.  Instead of reporting the line the exception occured
         # (which it correctly does with no args), it returns the line which
         # called the function which threw the exception.
         # We'll work around it by calling it twice
-        my($package, $filename, $line, $subname, $callsite) = _parent_stack_location();
+        my($package, $filename, $line, $subname) = _parent_stack_location();
 
         $uncaught_exception = Devel::Chitin::Exception->new(
             'package'   => $package,
@@ -549,7 +428,6 @@ $SIG{__DIE__} = sub {
             filename    => $filename,
             exception   => $exception,
             subroutine  => $subname,
-            callsite    => $callsite,
         );
         # After we fall off the end, the interpreter will try and exit,
         # triggering the END block that calls DB::fake::at_exit()
@@ -571,15 +449,6 @@ sub _execute_actions {
     }
 }
 
-sub fill_in_values_for_new_watch_exprs {
-    foreach my $detail ( @new_watch_exprs ) {
-        my($value) = _eval_in_program_context($detail->{expr}, 1);
-        $detail->{value} = $value;
-        push @watch_exprs, $detail;
-    }
-    @new_watch_exprs = ();
-}
-
 sub DB {
     return if (!$ready or $debugger_disabled or $in_debugger);
 
@@ -609,19 +478,16 @@ sub DB {
         filename    => $filename,
         line        => $line,
         subroutine  => $subroutine,
-        callsite    => scalar Devel::Chitin::Location::get_callsite(),
     );
 
     $_->notify_trace($current_location) foreach values(%trace_clients);
 
     _execute_actions($filename, $line);
 
-    goto RETURN_TO_DEBUGGED_PROGRAM if $no_stopping;
-
-    _evaluate_watch_exprs();
+    return if $no_stopping;
 
     if (! is_breakpoint($package, $filename, $line)) {
-        goto RETURN_TO_DEBUGGED_PROGRAM;
+        return;
     }
     $step_over_depth = undef;
 
@@ -643,32 +509,11 @@ sub DB {
 
         redo if ($finished || @pending_eval);
     }
-
-    fill_in_values_for_new_watch_exprs();
-
     Devel::Chitin::_do_each_client('notify_resumed', $current_location);
-
-    RETURN_TO_DEBUGGED_PROGRAM:
-
-    $_->notify_trace_resumed($current_location) foreach values(%trace_clients);
-
-    $previous_location = $current_location;
     undef $current_location;
-    Devel::Chitin::Stack::invalidate();
     restore();
 }
 
-BEGIN {
-    my $sub_serial = 1;
-    @Devel::Chitin::stack_serial = ( [ 'main::MAIN', $sub_serial++ ] );
-    %Devel::Chitin::eval_serial = ();
-
-    sub _allocate_sub_serial {
-        $sub_serial++;
-    }
-}
-
-
 sub sub {
     no strict 'refs';
     goto &$sub if (! $ready or index($sub, 'Devel::Chitin::StackTracker') == 0 or $debugger_disabled);
@@ -680,26 +525,13 @@ sub sub {
         unshift @AUTOLOAD_names, $caller_AUTOLOAD;
     }
     my $stack_tracker;
-    local @Devel::Chitin::stack_serial = @Devel::Chitin::stack_serial;
     unless ($in_debugger) {
+        my $tmp = $sub;
         $stack_depth++;
-        $stack_tracker = _new_stack_tracker(_allocate_sub_serial());
-
-        push(@Devel::Chitin::stack_serial, [ $sub, $$stack_tracker]);
+        $stack_tracker = _new_stack_tracker($tmp);
     }
 
-    my @rv;
-    if (wantarray) {
-        @rv = &$sub;
-    } elsif (defined wantarray) {
-        $rv[0] = &$sub;
-    } else {
-        &$sub;
-    }
-
-    delete $Devel::Chitin::eval_serial{$$stack_tracker} if $stack_tracker;
-
-    return wantarray ? @rv : $rv[0];
+    return &$sub;
 }
 
 sub _new_stack_tracker {
@@ -733,6 +565,7 @@ END {
     return if $debugger_disabled;
 
     $single=0;
+    $finished = 1;
     $in_debugger = 1;
 
     eval {
@@ -742,7 +575,6 @@ END {
             Devel::Chitin::_do_each_client('notify_program_exit');
         } else {
             Devel::Chitin::_do_each_client('notify_program_terminated', $?);
-            $finished = 1;
             # These two will trigger DB::DB and the event loop
             $in_debugger = 0;
             $single=1;
@@ -789,9 +621,6 @@ Devel::Chitin - Programmatic interface to the Perl debugging API
   CLIENT->is_breakable($file, $line);   # Return true if the line is executable
   CLIENT->stack();              # Return Devel::Chitin::Stack
   CLIENT->current_location();   # Where is the program stopped at?
-  CLIENT->next_statement([$scopes]);    # Return the next statement to execute
-  CLIENT->add_watchexpr($expr); # Add a new watch expression
-  CLIENT->remove_watchexpr($expr);  # Remove a watch expression
 
   # These methods are called by the debugging system at the appropriate time.
   # Base-class methods do nothing.  These methods must not block.
@@ -799,7 +628,6 @@ Devel::Chitin - Programmatic interface to the Perl debugging API
   CLIENT->poll($location);              # Return true if there is user input
   CLIENT->idle($location);              # Handle user interaction (can block)
   CLIENT->notify_trace($location);      # Called on each executable statement
-  CLIENT->notify_trace_resumed($location);  # Called before the program gets control after trace
   CLIENT->notify_stopped($location);    # Called when a break has occured
   CLIENT->notify_resumed($location);    # Called before the program gets control after a break
   CLIENT->notify_fork_parent($location,$pid);   # Called after fork() in parent
@@ -807,8 +635,6 @@ Devel::Chitin - Programmatic interface to the Perl debugging API
   CLIENT->notify_program_terminated($?);    # Called as the program is finishing 
   CLIENT->notify_program_exit();            # Called as the program is exiting
   CLIENT->notify_uncaught_exception($exc);  # Called after an uncaught exception
-  CLIENT->notify_watch_expr($location, $expr, $old, $new);
-                                        # Called when a watch expr changes
 
 =head1 DESCRIPTION
 
@@ -817,7 +643,7 @@ implementing debuggers, tracers, profilers, etc so they can all benefit from
 common code.
 
 Devel::Chitin is not a usable debugger per se.  It has no mechanism for interacting
-with a user such as reading command input or printing results.  Instead,
+with a user such as reading command input or printing retults.  Instead,
 clients of this API may call methods to inspect the debugged program state.
 The debugger core calls methods on clients when certain events occur, such
 as when the program is stopped by breakpoint or when the program exits.
@@ -851,9 +677,7 @@ C<attach> call.
 =item CLIENT->trace([1 | 0])
 
 Get or set the trace flag.  If trace is on, the client will get notified
-before every executable statement by having its C<notify_trace> method called,
-and before returning to the debugged program by having its
-C<notify_trace_resumed> method called.
+before every executable statement by having its C<notify_trace> method called.
 
 =item CLIENT->disable_debugger()
 
@@ -950,7 +774,7 @@ Return a list of loaded file names
 
 =item CLIENT->is_breakable($file, $line)
 
-Return true if the line has an executable statement.  Only lines with executable
+Return true if the line has an executable stament.  Only lines with executable
 statements may have breakpoints.  In particular, line containing only comments,
 whitespace or block delimiters are typically not breakable.
 
@@ -973,16 +797,6 @@ Return an instance of L<Devel::Chitin::Location> representing the currently
 stopped location in the debugged program.  This method returns undef if
 called when the debugged program is actively running.
 
-=item CLIENT->next_statement([$scope])
-
-Returns a string representing the next Perl statement to execute when control
-returns to the debugged program.  This involves inspecting the OpTree of the
-currently executing subroutine and deparsing it at the stopped location.
-Since the returned string is a reconstruction based on the OpTree, it may not
-match the original source code exactly.
-
-Requires the L<Devel::Callsite> module to be installed.
-
 =item CLIENT->file_source($filename)
 
 Return a list of strings containing the source code for a loaded file.
@@ -994,29 +808,6 @@ Return a list of strings containing the source code for a loaded file.
 See L<Devel::Chitin::Actionable> for documentation on setting breakpoints
 and actions.
 
-=head2 Watch expressions
-
-Watch expressions are evaluated before each statement in the program.  If a
-watched expression's value ever changes, the client that added the expression
-will be notified via its C<notify_watch_expr()> method.  These expressions
-are always evaluated in list context.  They are considered changed if the
-list's length changes, or if one of the elements has a different value
-when compared as strings.  This comparison is only shallow; it will not
-recurse into references or nested data structures.
-
-=over 4
-
-=item CLIENT->add_watch_expr($expression)
-
-Adds a new watch expression linked to the calling client.
-
-=item CLIENT->remove_watch_expr($expression)
-
-Remove a previously added watch expression.  Returns false if the expression
-was not previously added with C<add_watch_expr()>.
-
-=back
-
 =head2 CLIENT METHODS
 
 These methods exist in the base class, but only as empty stubs.  They are
@@ -1053,17 +844,12 @@ each executable statement.  The return value is ignored.  $location is an
 instance of L<Devel::Chitin::Location> indicating the next statement to be
 executed in the debugged program.
 
-=item CLIENT->notify_trace_resumed($location)
-
-If a client has turned on the trace flag, this method will be called before
-the debugged program regains control.  The return value is ignored.
-
 notify_trace() will be called only on clients that have requested tracing by
 calling CLIENT->trace(1).
 
 =item CLIENT->notify_stopped($location)
 
-This method is called when a breakpoint has occurred.  Its return value is
+This method is called when a breakpoint has occured.  Its return value is
 ignored.
 
 =item CLIENT->notify_resumed($location)
@@ -1109,19 +895,6 @@ not otherwise handled by the debugged program.  When an uncaught exception
 occurs, this method is called.  $exception is an instance of
 L<Devel::Chitin::Exception>.
 
-=item CLIENT->notify_watch_expr($location, $expr, $old, $new);
-
-Called when a client has added a watchexpr expression and its value has
-changed.  Since watch expressions are evaluated in list context, $old and
-$new are listrefs containing the previous and new values.
-
-The location reported is whichever program line was executing immediately
-prior to the current line.
-
-Note that this does not stop execution of the debugged program.  The
-notify_watch_expr() method should call C<CLIENT-E<gt>step> to trigger a
-breakpoint.
-
 =back
 
 =head1 BUGS
@@ -1129,13 +902,13 @@ breakpoint.
 As this is an extremely early release, this API should be considered
 experimental.  It was developed to extract the debugger-specific code
 from Devel::hdb.  I encourage others to make suggestions and submit bug
-reports so we can converge on a usable API quickly.
+reportss so we can converge on a usable API quickly.
 
 =head1 SEE ALSO
 
 L<Devel::Chitin::Location>, L<Devel::Chitin::Exception>,
 L<Devel::Chitin::Stack>, L<Devel::Chitin::Actionable>,
-L<Devel::Chitin::GetVarAtLevel>, L<Devel::Callsite>
+L<Devel::Chitin::GetVarAtLevel>
 
 The API for this module was inspired by L<DB>
 
