@@ -3,9 +3,8 @@ package Devel::Chitin::Actionable;
 use strict;
 use warnings;
 
-use Devel::Chitin::Version;
-
 use Digest::MD5 qw(md5);
+use List::Util;
 use Carp;
 
 sub new {
@@ -83,6 +82,11 @@ sub _insert {
     push @{$bp_info->{$type}}, $self;
 }
 
+sub _id {
+    my $self = shift;
+    md5(join('', @$self{'file', 'line', 'code'}, $self->type));
+}
+
 sub delete {
     my $self = shift;
 
@@ -132,17 +136,10 @@ sub delete {
  
 sub file    { return shift->{file} }
 sub line    { return shift->{line} }
+sub code    { return shift->{code} }
 sub once    { return shift->{once} }
 sub type    { my $class = shift;  $class = ref($class) || $class; die "$class didn't implement method type" }
 
-sub code    {
-    my $self = shift;
-    if (@_) {
-        $self->{code} = shift;
-    }
-    return $self->{code};
-}
-
 sub inactive {
     my $self = shift;
     if (@_) {
@@ -240,17 +237,13 @@ breakpoints for that file.  You may also filter by line, code and inactive.
 
 =item $bp->line
 
+=item $bp->code
+
 =item $bp->once
 
 Read-only accessors that return whatever values were used to create the
 breakpoint.
 
-=item $bp->code
-
-=item $bp->code($string)
-
-Mutator that retrieves the breakpoint's code condition, or sets it.
-
 =item $bp->inactive();
 
 =item $bp->inactive( 1 | 0);
@@ -266,7 +259,7 @@ Remove a breakpoint.  Deleted breakpoints will never trigger again.
 =head1 Actions
 
 Actions are a lot like breakpoints; they are associated with a file and line
-number, and they have code that runs before that line in the program is
+nummber, and they have code that runs before that line in the program is
 executed.  The difference is that the return value from the code is ignored.
 
 The code is evaluated in the context of the running program, so it can, for
@@ -297,17 +290,13 @@ that file.  You may also filter by line, code and inactive.
 
 =item $act->line
 
+=item $act->code
+
 =item $act->once
 
 Read-only accessors that return whatever values were used to create the
 action.
 
-=item $act->code
-
-=item $act->code($string);
-
-Mutator that retrieves the action's code, or set it.
-
 =item $act->inactive();
 
 =item $act->inactive( 1 | 0);
