@@ -3,8 +3,6 @@ package Devel::Chitin::Location;
 use strict;
 use warnings;
 
-use Devel::Chitin::Version;
-
 use Carp;
 
 sub new {
@@ -23,13 +21,10 @@ sub new {
 }
 
 sub _required_properties {
+    #qw( package filename line subroutine opaddr );
     qw( package filename line subroutine );
 }
 
-sub _optional_properties {
-    qw( callsite );
-}
-
 sub at_end {
     my $self = shift;
     return (($self->package eq 'Devel::Chitin::exiting')
@@ -47,7 +42,6 @@ sub current {
         if ($caller[3] eq 'DB::DB') {
             @props{'package','filename','line'} = @caller[0,1,2];
             $props{subroutine} = (caller($i+1))[3];
-            $props{callsite} = get_callsite($i);
             last;
         }
     }
@@ -57,10 +51,10 @@ sub current {
 sub _make_accessors {
     my $package = shift;
     my @accessor_names;
-    @accessor_names = ( $package->_required_properties, $package->_optional_properties );
+    @accessor_names = $package->_required_properties;
     if ($package ne __PACKAGE__) {
         # called as a class method by a subclass
-        my %base_class_accessors = map { $_ => 1 } (_required_properties(), _optional_properties());
+        my %base_class_accessors = map { $_ => 1 } _required_properties();
         @accessor_names = grep { ! $base_class_accessors{$_} } @accessor_names;
     }
  
@@ -72,19 +66,9 @@ sub _make_accessors {
     }
 }
 
-sub get_callsite { undef }
 
 BEGIN {
     __PACKAGE__->_make_accessors();
-
-    local $@;
-    my $site = eval { require Devel::Callsite && Devel::Callsite::callsite() };
-    if ($site) {
-        my $get_callsite_name = join('::', __PACKAGE__, 'get_callsite');
-        no strict 'refs';
-        no warnings 'redefine';
-        *$get_callsite_name = \&Devel::Callsite::callsite;
-    }
 }
 
 1;
@@ -117,8 +101,8 @@ This class is used to represent a location in the debugged program.
 
   Devel::Chitin::Location->new(%params)
 
-Construct a new instance.  The following parameters are accepted.  The values
-should be self-explanatory.  All parameters except callsite are required.
+Construct a new instnce.  The following parameters are accepted.  The values
+should be self-explanatory.  All parameters are required.
 
 =over 4
 
@@ -130,11 +114,6 @@ should be self-explanatory.  All parameters except callsite are required.
 
 =item subroutine
 
-=item callsite
-
-Represents the opcode address of the location as reported by Devel::Callsite::callsite().
-This value will only be valid if the optional module L<Devel::Callsite> is installed.
-
 =back
 
 Each construction parameter also has a read-only method to retrieve the value.
@@ -150,7 +129,7 @@ after the program has ended.
 
 =head1 SEE ALSO
 
-L<Devel::Chitin::Exception>, L<Devel::Chitin>, L<Devel::Callsite>
+L<Devel::Chitin::Exception>, L<Devel::Chitin>
 
 =head1 AUTHOR
 
