@@ -3,8 +3,6 @@ package Devel::Chitin::SubroutineLocation;
 use strict;
 use warnings;
 
-use Devel::Chitin::Version;
-
 use Carp;
 
 my @properties = qw(package subroutine line filename end code source source_line);
@@ -27,7 +25,7 @@ sub new {
 sub new_from_db_sub {
     my($class, $subname) = @_;
 
-    return () unless (defined($subname) and $DB::sub{$subname});
+    return () unless $DB::sub{$subname};
     my($filename, $line, $end) = $DB::sub{$subname} =~ m/(.*):(\d+)-(\d+)$/;
     my($source, $source_line) = $filename =~ m/\[(.*):(\d+)\]$/;
     my $glob = do {
